#include <qpainter.h>
#include <qpixmapcache.h>
#include <qpointarray.h>

#include <lights.h>


Lights::Lights(QWidget *parent)
    : QWidget(parent)
{
    m_color = Qt::red;
    m_diameter = 80;
}


QSize Lights::sizeHint() const
{
    return QSize(m_diameter, 3 * m_diameter);
}


void Lights::setDiameter(int diameter)
{
    m_diameter = diameter;
    update();
    updateGeometry();
}


void Lights::mousePressEvent(QMouseEvent *event)
{
    if (event->y() < m_diameter)
        m_color = Qt::red;
    else if (event->y() < 2 * m_diameter)
        m_color = Qt::yellow;
    else
        m_color = Qt::green;
    emit changed(m_color);
    update();
}


void Lights::paintEvent(QPaintEvent *)
{
    QString key = QString("lights:%1:%2")
                          .arg(m_color.name())
                          .arg(m_diameter);
    QPixmap pixmap;
    if (!QPixmapCache::find(key, pixmap)) {
        pixmap = generatePixmap();
        QPixmapCache::insert(key, pixmap);
    }
    bitBlt(this, 0, 0, &pixmap);
}


QPixmap Lights::generatePixmap()
{
    int w = m_diameter;
    int h = 3 * m_diameter;

    QPixmap pixmap(w, h);
    QPainter painter(&pixmap, this);
    painter.setBrush(Qt::darkGray);
    painter.drawRect(0, 0, w, h);
    painter.setBrush(m_color == Qt::red ? Qt::red : Qt::lightGray);
    painter.drawEllipse(0, 0, w, w);
    painter.setBrush(m_color == Qt::yellow ? Qt::yellow : Qt::lightGray);
    painter.drawEllipse(0, w, w, w);
    painter.setBrush(m_color == Qt::green ? Qt::green : Qt::lightGray);
    painter.drawEllipse(0, 2 * w, w, w);
    return pixmap;
}
